# Note: we will not include this as part of published secodadk

import json
from typing import Optional, Any

import httpx

from secodadk.ingestion_models import Resource, DeclaredLineage
from secodadk.strats.core import IngestResourcesStrategy, NetworkCallStrategy


class JSONLIngestResourcesStrategy(IngestResourcesStrategy):
    def __init__(self, *, resources_path: str, lineages_path: str):
        # Open file for writing
        self.resources_file = open(resources_path, "w")
        self.lineages_file = open(lineages_path, "w")

    def declare_resource(self, resource: Resource):
        self.resources_file.write(json.dumps(resource.model_dump(mode="json")) + "\n")

    def declare_lineage(self, lineage: DeclaredLineage):
        self.lineages_file.write(json.dumps(lineage.model_dump(mode="json")) + "\n")

    def finalize(self):
        self.resources_file.close()
        self.lineages_file.close()


class EgressServerNetworkCallStrategy(NetworkCallStrategy):
    def __init__(self, *, egress_server_url: str, egress_server_token: str):
        self.egress_server_url = egress_server_url
        self.egress_server_headers = {
            "Authorization": f"Bearer {egress_server_token}",
        }

    def make_request(
        self,
        method: str,
        url: str,
        *,
        params: Optional[dict] = None,
        headers: Optional[dict] = None,
        data: Optional[dict] = None,
        json: Optional[Any] = None,
        follow_redirects: bool = False,
        verify: bool | str = True,
    ):
        response = httpx.post(
            f"{self.egress_server_url}/egress/request/",
            headers=self.egress_server_headers,
            json={
                "method": method,
                "url": url,
                "params": params,
                "headers": headers,
                "data": data,
                "json": json,
                "follow_redirects": follow_redirects,
                "verify": verify,
            },
            timeout=600,
            verify=False,
        )

        if response.headers.get("Egress-Error"):
            raise httpx.HTTPError(response.text)

        return response



import abc
from typing import Optional, Any

from secodadk.ingestion_models import Resource, DeclaredLineage
from secodadk.strats.core import IngestResourcesStrategy, NetworkCallStrategy
from secodadk.strats.default import (
    DefaultIngestResourcesStrategy,
    DefaultNetworkCallStrategy,
)


class SecodaIntegration(abc.ABC):
    def __init__(
        self,
        *,
        credentials: dict,
        ingest_resources_strategy: Optional[IngestResourcesStrategy] = None,
        network_call_strategy: Optional[NetworkCallStrategy] = None,
    ):
        self.credentials: dict[str, str] = credentials
        self.ingest_resources_strategy: IngestResourcesStrategy = (
            ingest_resources_strategy or DefaultIngestResourcesStrategy()
        )
        self.network_call_strategy: NetworkCallStrategy = (
            network_call_strategy or DefaultNetworkCallStrategy()
        )

    def declare_resource(self, resource: Resource):
        """
        Declare a resource to be ingested by the integration.
        The resource can be a table, a dashboard, a column, etc.
        """
        self.ingest_resources_strategy.declare_resource(resource)

    def declare_lineage(self, lineage: DeclaredLineage):
        """
        Declare a lineage between resources
        """
        self.ingest_resources_strategy.declare_lineage(lineage)

    def finalize(self):
        self.ingest_resources_strategy.finalize()
        self.network_call_strategy.finalize()

    @abc.abstractmethod
    def extract(self):
        """
        The main extraction method of the integration.
        Your integration should implement this method to extract data from the source.

        You can use the http_get, http_post, http_put, http_patch, http_delete methods to make HTTP requests,
        and the declare_resource and declare_lineage methods to declare resources and lineages.

        Those resources and lineages will be ingested into Secoda.
        """
        pass

    def start(self):
        self.extract()
        self.finalize()

    def http_get(
        self,
        url: str,
        *,
        params: Optional[dict] = None,
        headers: Optional[dict] = None,
        follow_redirects: bool = False,
        verify: bool | str = True,
    ):
        return self.network_call_strategy.make_request(
            "GET",
            url,
            params=params,
            headers=headers,
            follow_redirects=follow_redirects,
            verify=verify,
        )

    def http_post(
        self,
        url: str,
        *,
        params: Optional[dict] = None,
        headers: Optional[dict] = None,
        data: Optional[dict] = None,
        json: Optional[Any] = None,
        follow_redirects: bool = False,
        verify: bool | str = True,
    ):
        return self.network_call_strategy.make_request(
            "POST",
            url,
            params=params,
            headers=headers,
            data=data,
            json=json,
            follow_redirects=follow_redirects,
            verify=verify,
        )

    def http_put(
        self,
        url: str,
        *,
        params: Optional[dict] = None,
        headers: Optional[dict] = None,
        data: Optional[dict] = None,
        json: Optional[Any] = None,
        follow_redirects: bool = False,
        verify: bool | str = True,
    ):
        return self.network_call_strategy.make_request(
            "PUT",
            url,
            params=params,
            headers=headers,
            data=data,
            json=json,
            follow_redirects=follow_redirects,
            verify=verify,
        )

    def http_patch(
        self,
        url: str,
        *,
        params: Optional[dict] = None,
        headers: Optional[dict] = None,
        data: Optional[dict] = None,
        json: Optional[Any] = None,
        follow_redirects: bool = False,
        verify: bool | str = True,
    ):
        return self.network_call_strategy.make_request(
            "PATCH",
            url,
            params=params,
            headers=headers,
            data=data,
            json=json,
            follow_redirects=follow_redirects,
            verify=verify,
        )

    def http_delete(
        self,
        url: str,
        *,
        params: Optional[dict] = None,
        headers: Optional[dict] = None,
        data: Optional[dict] = None,
        json: Optional[Any] = None,
        follow_redirects: bool = False,
        verify: bool | str = True,
    ):
        return self.network_call_strategy.make_request(
            "DELETE",
            url,
            params=params,
            headers=headers,
            data=data,
            json=json,
            follow_redirects=follow_redirects,
            verify=verify,
        )



import json
from typing import Optional, Any

import httpx

from secodadk.ingestion_models import Resource, DeclaredLineage
from secodadk.strats.core import IngestResourcesStrategy, NetworkCallStrategy


class DefaultIngestResourcesStrategy(IngestResourcesStrategy):
    def declare_resource(self, resource: Resource):
        print(json.dumps(resource.model_dump(mode="json")))

    def declare_lineage(self, lineage: DeclaredLineage):
        print(json.dumps(lineage.model_dump(mode="json")))

    def finalize(self):
        pass


class DefaultNetworkCallStrategy(NetworkCallStrategy):
    def make_request(
        self,
        method: str,
        url: str,
        *,
        params: Optional[dict] = None,
        headers: Optional[dict] = None,
        data: Optional[dict] = None,
        json: Optional[Any] = None,
        follow_redirects: bool = False,
        verify: bool | str = True,
    ):
        response = httpx.request(
            method,
            url,
            params=params,
            headers=headers,
            data=data,
            json=json,
            follow_redirects=follow_redirects,
            timeout=120,
            verify=verify,
        )

        return response



from datetime import datetime
from typing import Optional, Literal, Union

import pydantic
from pydantic import Field
from typing_extensions import Annotated


class Resource(pydantic.BaseModel):
    entity_type: str

    databuilder_id: str
    parent_databuilder_id: Optional[str] = None
    title: Optional[str] = ""
    description: Optional[str] = None
    definition: Optional[str] = None
    external_updated_at: Optional[datetime] = None
    native_type: Optional[str] = None

    # Column specific
    sort_order: Optional[int] = None
    type: Optional[str] = None
    is_pk: Optional[bool] = False
    hidden: Optional[bool] = False

    # Table specific, required for table
    schema: Optional[str] = None  # type: ignore
    database: Optional[str] = None

    # Dashboard specific
    group: Optional[str] = None

    # Chart specific
    product: Optional[str] = None


class InternalResource(pydantic.BaseModel):
    """
    InternalResource represents a resource that was declared and ingested by current integration.
    """

    type: Literal["internal_resource"] = "internal_resource"
    databuilder_id: str


class ExternalTable(pydantic.BaseModel):
    """
    ExternalTable represents a table in your workspace. It may or may not be part of the current integration.
    Secoda will use the following fields to match the table within the workspace.
    """

    type: Literal["external_table"] = "external_table"
    cluster: Optional[str] = None
    database: Optional[str] = None
    schema: Optional[str] = None
    table: str


class ExternalColumn(pydantic.BaseModel):
    """
    ExternalColumn represents a column in your workspace. It may or may not be part of the current integration.
    Secoda will use the following fields to match the column within the workspace.
    """

    type: Literal["external_column"] = "external_column"
    cluster: Optional[str] = None
    database: Optional[str] = None
    schema: Optional[str] = None
    table: str
    column: str


class TablesFromSQLQuery(pydantic.BaseModel):
    """
    TablesFromSQLQuery represents tables that are referenced in a SQL query.
    """

    type: Literal["tables_from_query"] = "tables_from_query"
    sql: str


# LineageID represents resource(s) in your workspace. The resource can be part of the current integration (InternalResource),
# or it can be a table or a column in your workspace (ExternalTable, ExternalColumn), or it can be a set of
# tables referenced in a SQL query (TablesFromSQLQuery).
LineageID = Annotated[
    Union[InternalResource, ExternalTable, ExternalColumn, TablesFromSQLQuery],
    Field(discriminator="type"),
]


class DeclaredLineage(pydantic.BaseModel):
    """
    A declared lineage represents a relationship between two resources in your workspace.
    """

    from_identifier: LineageID
    to_identifier: LineageID



import abc
from typing import Optional, Any

from httpx import Response

from secodadk.ingestion_models import Resource, DeclaredLineage


class IngestResourcesStrategy(abc.ABC):
    @abc.abstractmethod
    def declare_resource(self, resource: Resource):
        ...

    @abc.abstractmethod
    def declare_lineage(self, lineage: DeclaredLineage):
        ...

    @abc.abstractmethod
    def finalize(self):
        ...


class NetworkCallStrategy(abc.ABC):
    @abc.abstractmethod
    def make_request(
        self,
        method: str,
        url: str,
        *,
        params: Optional[dict] = None,
        headers: Optional[dict] = None,
        data: Optional[dict] = None,
        json: Optional[Any] = None,
        verify: bool | str = True,
        follow_redirects: bool = False,
    ) -> Response:
        ...

    def finalize(self):
        pass
